unit DKTUnit;

interface

const max_size = 50000;

type parapourDKT = record
     deltaH, Hlong, nu, pas : extended;
     xmaxKT : extended;
     nbiter : integer;
     end;

type TDKT_array = array[1..max_size,1..2] of extended;

var DKT_array : TDKT_array;

procedure calcul_DKT(parameters  : parapourDKT);
function interpoleKT(x,x1,x2,y1,y2 : extended) : extended;

implementation

function interpoleKT(x,x1,x2,y1,y2 : extended) : extended;
begin
    interpoleKT:=y1+(y2-y1)/(x2-x1)*(x-x1);
end;

procedure calcul_DKT(parameters : parapourDKT);
var tabKT : array[1..max_size] of extended; {pour stocker KT sans fluctuation}
    Pastot, nbiteration : integer;
    pasintegration, omegaL,delta : extended;
    xencours,dx, KTGtemp : extended;
    i,j : integer;
    t,integrale, taux, poubelle, old, proba : extended;
    compteur : integer;
    nu_fluctuation : extended;

begin
     randomize;
     with parameters do begin
          pastot:=trunc((xmaxKT)/pas+1);
          if pastot>max_size then pastot:=max_size-1;
          dx:=pas;
          nbiteration:=nbiter;
          pasintegration:=pas/10;
          omegaL:=Hlong*85.16229E-3; {omegaL en 2*PI*kHz}
          delta:=deltaH*85.16229E-3;
          nu_fluctuation:=nu;
     end;
     {on commence par calculer la KT avec champ sans fluctuations}
     xencours:=0;t:=0;integrale:=0;
     for i:=1 to pastot do
         begin
            xencours:=(i-1)*dx;
            DKT_array[i,1]:=xencours; DKT_array[i,2]:=1;
            t:=xencours;
            if omegal=0 then tabKT[i]:=1/3+2/3*(1-sqr(delta)*sqr(xencours))*exp(-sqr(delta)*sqr(xencours)/2)
               else begin
                      tabKT[i]:= 1-2*sqr(delta)/(omegal*omegal)*(1-exp(-delta*delta*sqr(xencours)/2)*cos(omegal*xencours))
                      +2*delta*delta*delta*delta/(omegal*omegal*omegal)*integrale;
                      for j:=1 to 10 do begin
                          integrale:=integrale+exp(-sqr(delta)*t*t/2)*sin(omegal*t)*dx/10;
                          t:=t+pasintegration;
                      end;
               end;
         end;
     if nu_fluctuation=0 then
             for i:=1 to Pastot do
                  DKT_array[i,2]:=tabKT[i]
     else begin
            compteur:=0;
            while compteur<=nbiteration do begin
                 compteur:=compteur+1;
                 old:=0.0;
                 t:=0.0;
                 taux:=0.0;
                 poubelle:=1;

                 for i:=1 to Pastot do
                 begin
                      t:=t+dx;
                      taux:=taux+dx;
                      if old=0 then proba:=random(100)/100;
                      j:=1;
                      while ((taux>=DKT_array[j,1]) and (j<pastot)) do j:=j+1;
                      KTGtemp:=interpoleKT(taux,DKT_array[j-1,1],DKT_array[j,1],tabKT[j-1],tabKT[j]);
                      DKT_array[i,2]:=DKT_array[i,2]*(compteur-1)/compteur +KTGtemp*poubelle/compteur;

                      if exp(-nu_fluctuation*taux)<proba then begin
                        poubelle:=poubelle*KTGtemp;
                        taux:=0;
                        old:=0;
                      end
                      else old:=1;
                 end;
            end;
         end;
end;


end.
